@@ -11,20 +11,21 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized once );
 
-our $VERSION = '1.867_01';
+our $VERSION = '1.822';
 
 use base 'MCE::Mutex';
 use Fcntl ':flock';
 use Carp ();
 
-my $tid = $INC{'threads.pm'} ? threads->tid() : 0;
+my $has_threads = $INC{'threads.pm'} ? 1 : 0;
+my $tid = $has_threads ? threads->tid()  : 0;
 
 sub CLONE {
-    $tid = threads->tid() if $INC{'threads.pm'};
+    $tid = threads->tid() if $has_threads;
 }
 
 sub DESTROY {
-    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
 
     $obj->unlock(), close(delete $obj->{_fh}) if $obj->{ $pid };
 
@@ -34,7 +35,8 @@ sub DESTROY {
 }
 
 sub _open {
-    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
+
     return if exists $obj->{ $pid };
 
     open $obj->{_fh}, '+>>:raw:stdio', $obj->{path}
@@ -49,11 +51,14 @@ sub _open {
 ##
 ###############################################################################
 
-my ($id, $prog_name) = (0);
+my ($id, $prog_name, $is_winenv) = (0);
 
-$prog_name =  $0;
-$prog_name =~ s{^.*[\\/]}{}g;
-$prog_name =  'perl' if ($prog_name eq '-e' || $prog_name eq '-');
+BEGIN {
+    $prog_name =  $0;
+    $prog_name =~ s{^.*[\\/]}{}g;
+    $prog_name =  'perl' if ($prog_name eq '-e' || $prog_name eq '-');
+    $is_winenv =  ($^O =~ /mswin|mingw|msys|cygwin/i) ? 1 : 0;
+}
 
 sub new {
     my ($class, %obj) = (@_, impl => 'Flock');
@@ -62,13 +67,13 @@ sub new {
         my ($pid, $tmp_dir, $tmp_file) = ( abs($$) );
 
         if ($ENV{TEMP} && -d $ENV{TEMP} && -w _) {
-            if ($^O =~ /mswin|mingw|msys|cygwin/i) {
+            if ($is_winenv) {
                 $tmp_dir  = $ENV{TEMP};
                 $tmp_dir .= ($^O eq 'MSWin32') ? "\\Perl-MCE" : "/Perl-MCE";
                 mkdir $tmp_dir unless (-d $tmp_dir);
             }
             else {
-                $tmp_dir = $ENV{TEMP};
+                $tmp_dir  = $ENV{TEMP};
             }
         }
         elsif ($ENV{TMPDIR} && -d $ENV{TMPDIR} && -w _) {
@@ -78,10 +83,10 @@ sub new {
             $tmp_dir = '/tmp';
         }
         else {
-            Carp::croak("No writable dir found for a temp file");
+            Carp::croak("no writable dir found for temp file");
         }
 
-        $id++, $tmp_dir =~ s{[\\/]$}{};
+        $id++, $tmp_dir =~ s{/$}{};
 
         # remove tainted'ness from $tmp_dir
         if ($^O eq 'MSWin32') {
@@ -90,31 +95,24 @@ sub new {
             ($tmp_file) = "$tmp_dir/$prog_name.$pid.$tid.$id" =~ /(.*)/;
         }
 
-        $obj{_init} = $tid ? $$ .'.'. $tid : $$;
+        $obj{_init} = $has_threads ? $$ .'.'. $tid : $$;
         $obj{ path} = $tmp_file.'.lock';
-
-        # test open
-        open my $fh, '+>>:raw:stdio', $obj{path}
-            or Carp::croak("Could not create temp file $obj{path}: $!");
-
-        close $fh;
-
-        # set permission
-        chmod 0600, $obj{path};
     }
-    else {
-        # test open
-        open my $fh, '+>>:raw:stdio', $obj{path}
-            or Carp::croak("Could not obtain flock on file $obj{path}: $!");
 
-        close $fh;
-    }
+    # test open
+    open my $fh, '+>>:raw:stdio', $obj{path}
+        or Carp::croak("Could not create temp file $obj{path}: $!");
+
+    close $fh;
+
+    # update permission
+    chmod 0600, $obj{path} if $obj{_init};
 
     return bless(\%obj, $class);
 }
 
 sub lock {
-    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
     $obj->_open() unless exists $obj->{ $pid };
 
     flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1
@@ -126,7 +124,7 @@ sub lock {
 *lock_exclusive = \&lock;
 
 sub lock_shared {
-    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
     $obj->_open() unless exists $obj->{ $pid };
 
     flock ($obj->{_fh}, LOCK_SH), $obj->{ $pid } = 1
@@ -136,7 +134,7 @@ sub lock_shared {
 }
 
 sub unlock {
-    my ($pid, $obj) = ($tid ? $$ .'.'. $tid : $$, @_);
+    my ($pid, $obj) = ($has_threads ? $$ .'.'. $tid : $$, @_);
 
     flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0
         if $obj->{ $pid };
@@ -145,19 +143,17 @@ sub unlock {
 }
 
 sub synchronize {
-    my ($pid, $obj, $code, @ret) = ($tid ? $$ .'.'. $tid : $$, shift, shift);
-    return unless ref($code) eq 'CODE';
+    my ($pid, $obj, $code, @ret) = (
+        $has_threads ? $$ .'.'. $tid : $$, shift, shift
+    );
+
+    return if ref($code) ne 'CODE';
 
     $obj->_open() unless exists $obj->{ $pid };
 
     # lock, run, unlock - inlined for performance
-    flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1
-        unless $obj->{ $pid };
-
-    (defined wantarray)
-      ? @ret = wantarray ? $code->(@_) : scalar $code->(@_)
-      : $code->(@_);
-
+    flock ($obj->{_fh}, LOCK_EX), $obj->{ $pid } = 1 unless $obj->{ $pid };
+    defined wantarray ? @ret = $code->(@_) : $code->(@_);
     flock ($obj->{_fh}, LOCK_UN), $obj->{ $pid } = 0;
 
     return wantarray ? @ret : $ret[-1];
@@ -181,33 +177,11 @@ MCE::Mutex::Flock - Mutex locking via Fcntl
 
 =head1 VERSION
 
-This document describes MCE::Mutex::Flock version 1.867_01
+This document describes MCE::Mutex::Flock version 1.822
 
 =head1 DESCRIPTION
 
-A Fcntl implementation for C<MCE::Mutex>.
-
-The API is described in L<MCE::Mutex>.
-
-=over 3
-
-=item new
-
-=item lock
-
-=item lock_exclusive
-
-=item lock_shared
-
-=item unlock
-
-=item synchronize
-
-=item enter
-
-=item timedwait
-
-=back
+A Fcntl implementation for L<MCE::Mutex>. See documentation there.
 
 =head1 AUTHOR
 
